/*
 * gtk_libvlc_quvi_exemple: this program shows the use of quvi along with libvlc
 *
 * Copyright (C) 2011 Paul Kocialkowski <contact@paulk.fr>
 *
 * libsamsung-ipc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libsamsung-ipc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libsamsung-ipc.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <vlc/vlc.h>
#include <quvi/quvi.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#define APP_NAME	"gtk_libvlc_quvi_exemple"
#define APP_VERSION	"0.1-gci"

libvlc_instance_t *vlc_instance;
libvlc_media_player_t *vlc_media_player;
libvlc_event_manager_t *vlc_event_manager;
bool vlc_media_player_stopped;

quvi_t quvi_session;
quvi_media_t quvi_media;
QUVIcode rc;

char *quvi_media_url = NULL;
char *quvi_media_title = NULL;
char *quvi_media_suffix = NULL;

GtkWidget *input_status_image = NULL;
GtkWidget *input_button = NULL;
GtkWidget *input_entry = NULL;
GtkWidget *video_window = NULL;
GtkWidget *video_play_button;
GtkWidget *video_stop_button;
GtkWidget *video_seek_scale = NULL;
GtkWidget *video_volume_scale_button = NULL;
GtkWidget *video_watch_button = NULL;
GtkWidget *video_dl_button = NULL;

/*
    media_player = libvlc_media_player_new(vlc_inst);
    g_signal_connect(G_OBJECT(player_widget), "realize", G_CALLBACK(player_widget_on_realize), NULL);
*/

void main_quit(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();

	libvlc_release(vlc_instance);
}

void video_quit(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(GTK_WINDOW(video_window));
	video_window = NULL;

	libvlc_media_player_release(vlc_media_player);

	gtk_button_set_label(GTK_BUTTON(video_watch_button), "Watch");
}

void vlc_play(void)
{
	libvlc_media_player_play(vlc_media_player);
	gtk_button_set_image(GTK_BUTTON(video_play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON));
	vlc_media_player_stopped = false;
}

void vlc_pause_toogle(void)
{
	libvlc_media_player_pause(vlc_media_player);
	int is_playing = libvlc_media_player_is_playing(vlc_media_player);

	gtk_button_set_image(GTK_BUTTON(video_play_button), is_playing
		? gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON)
		: gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON)
	);
}

void vlc_stopped(void)
{
	vlc_media_player_stopped = true;
	libvlc_media_player_set_position(vlc_media_player, 0);

	gtk_range_set_value(GTK_RANGE(video_seek_scale), 0);
	gtk_button_set_image(GTK_BUTTON(video_play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
}

void vlc_stop(void)
{
	libvlc_media_player_set_pause(vlc_media_player, 1);
	libvlc_media_player_stop(vlc_media_player);

	vlc_stopped();
}

void stop_clicked(GtkWidget *widget, gpointer data)
{
	vlc_stop();
}

void play_clicked(GtkWidget *widget, gpointer data)
{
	if(vlc_media_player_stopped)
	{
		libvlc_media_player_stop(vlc_media_player);
		vlc_play();
	}
	else
	{
		vlc_pause_toogle();
	}
}

void volume_init(void)
{
	int volume = libvlc_audio_get_volume(vlc_media_player);
	gtk_scale_button_set_value(GTK_SCALE_BUTTON(video_volume_scale_button), volume/100);
}

void end_reached(const libvlc_event_t *event, void *data)
{
	libvlc_media_player_set_pause(vlc_media_player, 1);
	vlc_stopped();
}

void position_changed(const libvlc_event_t *event, void *data)
{
	float position = libvlc_media_player_get_position(vlc_media_player);
	gtk_range_set_value(GTK_RANGE(video_seek_scale), position);
}

void draw_realize(GtkWidget *widget, gpointer data)
{
	libvlc_media_t *vlc_media;

	libvlc_media_player_set_xwindow(vlc_media_player, GDK_WINDOW_XID(gtk_widget_get_window(widget)));

	if(quvi_media_url == NULL || quvi_media_title == NULL)
	{
		video_quit(NULL, NULL);
		return;
	}

	vlc_media = libvlc_media_new_location(vlc_instance, quvi_media_url);
	libvlc_media_set_meta(vlc_media, libvlc_meta_Title, quvi_media_title);
	libvlc_media_player_set_media(vlc_media_player, vlc_media);
	//Add fullscreen buttons
//	gtk_window_fullscreen(video_window);
	vlc_play();
	libvlc_media_release(vlc_media);

	volume_init();
}

gboolean volume_value_changed(GtkScaleButton *button, gdouble value, gpointer user_data)
{
	float volume = gtk_scale_button_get_value(GTK_SCALE_BUTTON(video_volume_scale_button));
	libvlc_audio_set_volume(vlc_media_player, volume * 100);
}

gboolean seek_value_changed(GtkRange *range, GtkScrollType scroll, gdouble value, gpointer data)
{
	float position = (float) value;
	if(position > 1)
		position = 1;
	if(position < 0)
		position = 0;

	libvlc_media_player_set_position(vlc_media_player, position);

	return FALSE;
}

void watch_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *video_player_draw;
	GtkWidget *video_vbox;
	GtkWidget *video_hbox;
	GtkWidget *video_buttons_hbox;
	GdkColor color;

	if(video_window != NULL)
	{
		video_quit(NULL, NULL);
	}
	else
	{
		video_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_default_size(GTK_WINDOW(video_window), 640, 480);
		gtk_window_set_title(GTK_WINDOW(video_window), APP_NAME);
		g_signal_connect(G_OBJECT(video_window), "destroy", G_CALLBACK(video_quit), NULL);

		video_vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(video_window), video_vbox);

		video_player_draw = gtk_drawing_area_new();
		gdk_color_parse ("black", &color);
		gtk_widget_modify_bg(video_player_draw, GTK_STATE_NORMAL, &color);
		gtk_box_pack_start(GTK_BOX(video_vbox), video_player_draw, TRUE, TRUE, 0);

		video_hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(video_vbox), video_hbox, FALSE, FALSE, 0);

		video_buttons_hbox = gtk_hbutton_box_new();
		gtk_button_box_set_layout(GTK_BUTTON_BOX(video_buttons_hbox), GTK_BUTTONBOX_START);
//		gtk_box_set_spacing(GTK_BOX(video_buttons_hbox), 10);
		gtk_box_pack_start(GTK_BOX(video_hbox), video_buttons_hbox, FALSE, FALSE, 0);

		video_play_button = gtk_button_new();
		gtk_button_set_image(GTK_BUTTON(video_play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
		g_signal_connect(G_OBJECT(video_play_button), "clicked", G_CALLBACK(play_clicked), NULL);
		gtk_box_pack_start(GTK_BOX(video_buttons_hbox), video_play_button, FALSE, FALSE, 0);

		video_stop_button = gtk_button_new();
		gtk_button_set_image(GTK_BUTTON(video_stop_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_STOP, GTK_ICON_SIZE_BUTTON));
		g_signal_connect(G_OBJECT(video_stop_button), "clicked", G_CALLBACK(stop_clicked), NULL);
		gtk_box_pack_start(GTK_BOX(video_buttons_hbox), video_stop_button, FALSE, FALSE, 0);

		video_seek_scale = gtk_hscale_new_with_range(0, 1, 0.01);
		gtk_scale_set_draw_value(GTK_SCALE(video_seek_scale), FALSE);
		g_signal_connect(G_OBJECT(video_seek_scale), "change-value", G_CALLBACK(seek_value_changed), NULL);
		gtk_box_pack_start(GTK_BOX(video_hbox), video_seek_scale, TRUE, TRUE, 0);

		video_volume_scale_button = gtk_volume_button_new();
		g_signal_connect(G_OBJECT(video_volume_scale_button), "value-changed", G_CALLBACK(volume_value_changed), NULL);
		gtk_box_pack_start(GTK_BOX(video_hbox), video_volume_scale_button, FALSE, FALSE, 0);

		gtk_button_set_label(GTK_BUTTON(video_watch_button), "Close");

		vlc_media_player = libvlc_media_player_new(vlc_instance);
		vlc_event_manager = libvlc_media_player_event_manager(vlc_media_player);
		libvlc_event_attach(vlc_event_manager, libvlc_MediaPlayerPositionChanged, position_changed, NULL);
		libvlc_event_attach(vlc_event_manager, libvlc_MediaPlayerEndReached, end_reached, NULL);

		g_signal_connect(G_OBJECT(video_player_draw), "realize", G_CALLBACK(draw_realize), NULL);

		gtk_widget_show_all(video_window);
	}
}

void input_clicked(GtkWidget *widget, gpointer data)
{
	char *input_entry_text;
	char *url;
	char *title;
	char *suffix;

	input_entry_text = gtk_entry_get_text(GTK_ENTRY(input_entry));
	if(input_entry_text == NULL)
		return;

	printf("%s\n", input_entry_text);

	rc = quvi_parse(quvi_session, input_entry_text, &quvi_media);

	if(rc != QUVI_OK)
		goto exit_failure;

	quvi_getprop(quvi_media, QUVIPROP_MEDIAURL, &url);
	quvi_getprop(quvi_media, QUVIPROP_PAGETITLE, &title);
	quvi_getprop(quvi_media, QUVIPROP_FILESUFFIX, &suffix);

	if(url == NULL || title == NULL || suffix == NULL)
		goto exit_failure;

	gtk_image_set_from_stock(input_status_image, GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
	gtk_widget_set_sensitive(video_watch_button, TRUE);
	gtk_widget_set_sensitive(video_dl_button, TRUE);

	if(quvi_media_url != NULL)
		free(quvi_media_url);
	quvi_media_url = malloc(strlen(url) + 1);
	memcpy(quvi_media_url, url, strlen(url) + 1);

	if(quvi_media_title != NULL)
		free(quvi_media_title);
	quvi_media_title = malloc(strlen(title) + 1);
	memcpy(quvi_media_title, title, strlen(title) + 1);

	if(quvi_media_suffix != NULL)
		free(quvi_media_suffix);
	quvi_media_suffix = malloc(strlen(suffix) + 1);
	memcpy(quvi_media_suffix, suffix, strlen(suffix) + 1);

	goto exit;

exit_failure:
	gtk_image_set_from_stock(input_status_image, GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
	gtk_widget_set_sensitive(video_watch_button, FALSE);
	gtk_widget_set_sensitive(video_dl_button, FALSE);

exit:
	quvi_parse_close(&quvi_media);
}

int main(int argc, char *argv[])
{
	GtkWidget *main_window;

	GtkWidget *main_vbox;
	GtkWidget *main_instruct_label;
	GtkWidget *input_hbox;
	GtkWidget *video_hbox;

	vlc_instance = libvlc_new(0, NULL);

	rc = quvi_init(&quvi_session);

	if(rc != QUVI_OK)
		return 1;

	quvi_setopt(quvi_session, QUVIOPT_FORMAT, "default");
	quvi_setopt(quvi_session, QUVIOPT_CATEGORY, QUVIPROTO_HTTP);

	gtk_init(&argc, &argv);

	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(main_window), APP_NAME);
	gtk_window_set_default_size(GTK_WINDOW(main_window), 400, 100);
	g_signal_connect(G_OBJECT(main_window), "destroy", G_CALLBACK(main_quit), NULL);

	main_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);

	gtk_container_add(GTK_CONTAINER(main_window), main_vbox);

	main_instruct_label = gtk_label_new("Please paste the video website URL:");
	gtk_box_pack_start(GTK_BOX(main_vbox), main_instruct_label, TRUE, FALSE, 0);

	input_hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_set_spacing(GTK_BOX(input_hbox), 10);
	gtk_box_pack_start(GTK_BOX(main_vbox), input_hbox, TRUE, FALSE, 0);

	input_status_image = gtk_image_new();
	gtk_image_set_from_stock(GTK_IMAGE(input_status_image), GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);

	gtk_box_pack_start(GTK_BOX(input_hbox), input_status_image, FALSE, FALSE, 0);

	input_entry = gtk_entry_new();
	//TODO: connect to signal preedit-changed
	gtk_box_pack_start(GTK_BOX(input_hbox), input_entry, TRUE, TRUE, 0);

	input_button = gtk_button_new_from_stock(GTK_STOCK_OK);
	gtk_box_pack_start(GTK_BOX(input_hbox), input_button, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(input_button), "clicked", G_CALLBACK(input_clicked), NULL);

	video_hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(video_hbox), GTK_BUTTONBOX_CENTER);
	gtk_box_set_spacing(GTK_BOX(video_hbox), 10);
	gtk_box_pack_start(GTK_BOX(main_vbox), video_hbox, FALSE, FALSE, 0);

	video_watch_button = gtk_button_new_with_label("Watch");
	gtk_button_set_image(GTK_BUTTON(video_watch_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
	g_signal_connect(G_OBJECT(video_watch_button), "clicked", G_CALLBACK(watch_clicked), NULL);
	gtk_widget_set_sensitive(video_watch_button, FALSE);
	gtk_box_pack_start(GTK_BOX(video_hbox), video_watch_button, FALSE, FALSE, 0);
/*
	video_dl_button = gtk_button_new_with_label("Download");
	gtk_button_set_image(GTK_BUTTON(video_dl_button), gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_BUTTON));
//	g_signal_connect(G_OBJECT(video_watch_button), "clicked", G_CALLBACK(), NULL);
	gtk_widget_set_sensitive(video_dl_button, FALSE);
	gtk_box_pack_start(GTK_BOX(video_hbox), video_dl_button, FALSE, FALSE, 0);
*/

	gtk_widget_show_all(main_window);

	gtk_main();	

	return 0;
}
